@@ -29,7 +29,10 @@ _gcry_compat_identification (void)
     "\n\n"
     "This is Libgcrypt " PACKAGE_VERSION " - The GNU Crypto Library\n"
     "Copyright 2000, 2002, 2003, 2004, 2007, 2008, 2009,\n"
-    "          2010, 2011 Free Software Foundation, Inc.\n"
+    "          2010, 2011, 2012 Free Software Foundation, Inc.\n"
+    "Copyright 2012, 2013 g10 Code GmbH\n"
+    "\n"
+    "(" BUILD_REVISION " " BUILD_TIMESTAMP ")\n"
     "\n\n";
   return blurb;
 }
