@@ -32,6 +32,7 @@
 
 #ifdef _GCRYPT_IN_LIBGCRYPT
 # include "../src/gcrypt.h"
+# include "../compat/libcompat.h"
 #else
 # include <gcrypt.h>
 #endif
@@ -47,6 +48,15 @@ static int large_buffers;
 /* Number of cipher repetitions.  */
 static int cipher_repetitions;
 
+/* Number of hash repetitions.  */
+static int hash_repetitions;
+
+/* Alignment of the buffers.  */
+static int buffer_alignment;
+
+/* Whether to include the keysetup in the cipher timings.  */
+static int cipher_with_keysetup;
+
 /* Whether fips mode was active at startup.  */
 static int in_fips_mode;
 
@@ -287,9 +297,15 @@ static void
 start_timer (void)
 {
 #ifdef _WIN32
+#ifdef __MINGW32CE__
+  GetThreadTimes (GetCurrentThread (),
+                   &started_at.creation_time, &started_at.exit_time,
+                   &started_at.kernel_time, &started_at.user_time);
+#else
   GetProcessTimes (GetCurrentProcess (),
                    &started_at.creation_time, &started_at.exit_time,
                    &started_at.kernel_time, &started_at.user_time);
+#endif
   stopped_at = started_at;
 #else
   struct tms tmp;
@@ -303,9 +319,15 @@ static void
 stop_timer (void)
 {
 #ifdef _WIN32
+#ifdef __MINGW32CE__
+  GetThreadTimes (GetCurrentThread (),
+                   &stopped_at.creation_time, &stopped_at.exit_time,
+                   &stopped_at.kernel_time, &stopped_at.user_time);
+#else
   GetProcessTimes (GetCurrentProcess (),
                    &stopped_at.creation_time, &stopped_at.exit_time,
                    &stopped_at.kernel_time, &stopped_at.user_time);
+#endif
 #else
   struct tms tmp;
 
@@ -339,6 +361,18 @@ elapsed_time (void)
 }
 
 
+static void
+progress_cb (void *cb_data, const char *what, int printchar,
+             int current, int total)
+{
+  (void)cb_data;
+
+  fprintf (stderr, PGM ": progress (%s %c %d %d)\n",
+           what, printchar, current, total);
+  fflush (stderr);
+}
+
+
 static void
 random_bench (int very_strong)
 {
@@ -375,8 +409,13 @@ md_bench ( const char *algoname )
 {
   int algo;
   gcry_md_hd_t hd;
-  int i;
-  char buf[1000];
+  int i, j, repcount;
+  char buf_base[1000+15];
+  size_t bufsize = 1000;
+  char *buf;
+  char *largebuf_base;
+  char *largebuf;
+  char digest[512/8];
   gcry_error_t err = GPG_ERR_NO_ERROR;
 
   if (!algoname)
@@ -389,6 +428,8 @@ md_bench ( const char *algoname )
       return;
     }
 
+  buf = buf_base + ((16 - ((size_t)buf_base & 0x0f)) % buffer_alignment);
+
   algo = gcry_md_map_name (algoname);
   if (!algo)
     {
@@ -403,36 +444,76 @@ md_bench ( const char *algoname )
       exit (1);
     }
 
-  for (i=0; i < sizeof buf; i++)
+  for (i=0; i < bufsize; i++)
     buf[i] = i;
 
   printf ("%-12s", gcry_md_algo_name (algo));
 
   start_timer ();
-  for (i=0; i < 1000; i++)
-    gcry_md_write (hd, buf, sizeof buf);
+  for (repcount=0; repcount < hash_repetitions; repcount++)
+    for (i=0; i < 1000; i++)
+      gcry_md_write (hd, buf, bufsize);
   gcry_md_final (hd);
   stop_timer ();
   printf (" %s", elapsed_time ());
+  fflush (stdout);
 
   gcry_md_reset (hd);
   start_timer ();
-  for (i=0; i < 10000; i++)
-    gcry_md_write (hd, buf, sizeof buf/10);
+  for (repcount=0; repcount < hash_repetitions; repcount++)
+    for (i=0; i < 10000; i++)
+      gcry_md_write (hd, buf, bufsize/10);
   gcry_md_final (hd);
   stop_timer ();
   printf (" %s", elapsed_time ());
+  fflush (stdout);
 
   gcry_md_reset (hd);
   start_timer ();
-  for (i=0; i < 1000000; i++)
-    gcry_md_write (hd, "", 1);
+  for (repcount=0; repcount < hash_repetitions; repcount++)
+    for (i=0; i < 1000000; i++)
+      gcry_md_write (hd, buf, 1);
   gcry_md_final (hd);
   stop_timer ();
   printf (" %s", elapsed_time ());
+  fflush (stdout);
+
+  start_timer ();
+  for (repcount=0; repcount < hash_repetitions; repcount++)
+    for (i=0; i < 1000; i++)
+      for (j=0; j < bufsize; j++)
+        gcry_md_putc (hd, buf[j]);
+  gcry_md_final (hd);
+  stop_timer ();
+  printf (" %s", elapsed_time ());
+  fflush (stdout);
 
   gcry_md_close (hd);
+
+  /* Now 100 hash operations on 10000 bytes using the fast function.
+     We initialize the buffer so that all memory pages are committed
+     and we have repeatable values.  */
+  if (gcry_md_get_algo_dlen (algo) > sizeof digest)
+    die ("digest buffer too short\n");
+
+  largebuf_base = malloc (10000+15);
+  if (!largebuf_base)
+    die ("out of core\n");
+  largebuf = (largebuf_base
+              + ((16 - ((size_t)largebuf_base & 0x0f)) % buffer_alignment));
+
+  for (i=0; i < 10000; i++)
+    largebuf[i] = i;
+  start_timer ();
+  for (repcount=0; repcount < hash_repetitions; repcount++)
+    for (i=0; i < 100; i++)
+      gcry_md_hash_buffer (algo, digest, largebuf, 10000);
+  stop_timer ();
+  printf (" %s", elapsed_time ());
+  free (largebuf_base);
+
   putchar ('\n');
+  fflush (stdout);
 }
 
 static void
@@ -445,15 +526,16 @@ cipher_bench ( const char *algoname )
   int keylen, blklen;
   char key[128];
   char *outbuf, *buf;
+  char *raw_outbuf, *raw_buf;
   size_t allocated_buflen, buflen;
   int repetitions;
   static struct { int mode; const char *name; int blocked; } modes[] = {
-    { GCRY_CIPHER_MODE_ECB, "ECB", 1 },
-    { GCRY_CIPHER_MODE_CBC, "CBC", 1 },
-    { GCRY_CIPHER_MODE_CFB, "CFB", 0 },
-    { GCRY_CIPHER_MODE_OFB, "OFB", 0 },
-    { GCRY_CIPHER_MODE_CTR, "CTR", 0 },
-    { GCRY_CIPHER_MODE_STREAM, "STREAM", 0 },
+    { GCRY_CIPHER_MODE_ECB, "   ECB/Stream", 1 },
+    { GCRY_CIPHER_MODE_CBC, "      CBC", 1 },
+    { GCRY_CIPHER_MODE_CFB, "      CFB", 0 },
+    { GCRY_CIPHER_MODE_OFB, "      OFB", 0 },
+    { GCRY_CIPHER_MODE_CTR, "      CTR", 0 },
+    { GCRY_CIPHER_MODE_STREAM, "", 0 },
     {0}
   };
   int modeidx;
@@ -480,8 +562,12 @@ cipher_bench ( const char *algoname )
     }
   repetitions *= cipher_repetitions;
 
-  buf = gcry_xmalloc (allocated_buflen);
-  outbuf = gcry_xmalloc (allocated_buflen);
+  raw_buf = gcry_xmalloc (allocated_buflen+15);
+  buf = (raw_buf
+         + ((16 - ((size_t)raw_buf & 0x0f)) % buffer_alignment));
+  outbuf = raw_outbuf = gcry_xmalloc (allocated_buflen+15);
+  outbuf = (raw_outbuf
+            + ((16 - ((size_t)raw_outbuf & 0x0f)) % buffer_alignment));
 
   if (!header_printed)
     {
@@ -489,11 +575,13 @@ cipher_bench ( const char *algoname )
         printf ("Running each test %d times.\n", cipher_repetitions);
       printf ("%-12s", "");
       for (modeidx=0; modes[modeidx].mode; modeidx++)
-        printf (" %-15s", modes[modeidx].name );
+        if (*modes[modeidx].name)
+          printf (" %-15s", modes[modeidx].name );
       putchar ('\n');
       printf ("%-12s", "");
       for (modeidx=0; modes[modeidx].mode; modeidx++)
-        printf (" ---------------" );
+        if (*modes[modeidx].name)
+          printf (" ---------------" );
       putchar ('\n');
       header_printed = 1;
     }
@@ -536,10 +624,7 @@ cipher_bench ( const char *algoname )
     {
       if ((blklen > 1 && modes[modeidx].mode == GCRY_CIPHER_MODE_STREAM)
           | (blklen == 1 && modes[modeidx].mode != GCRY_CIPHER_MODE_STREAM))
-        {
-          printf ("                " );
-          continue;
-        }
+        continue;
 
       for (i=0; i < sizeof buf; i++)
         buf[i] = i;
@@ -550,30 +635,46 @@ cipher_bench ( const char *algoname )
           fprintf (stderr, PGM ": error opening cipher `%s'\n", algoname);
           exit (1);
         }
-      
-      err = gcry_cipher_setkey (hd, key, keylen);
-      if (err)
-      { 
-          fprintf (stderr, "gcry_cipher_setkey failed: %s\n",
-		   gpg_strerror (err));
-          gcry_cipher_close (hd);
-          exit (1);
+
+      if (!cipher_with_keysetup)
+        {
+          err = gcry_cipher_setkey (hd, key, keylen);
+          if (err)
+            {
+              fprintf (stderr, "gcry_cipher_setkey failed: %s\n",
+                       gpg_strerror (err));
+              gcry_cipher_close (hd);
+              exit (1);
+            }
         }
 
       buflen = allocated_buflen;
       if (modes[modeidx].blocked)
         buflen = (buflen / blklen) * blklen;
-      
+
       start_timer ();
       for (i=err=0; !err && i < repetitions; i++)
-        err = gcry_cipher_encrypt ( hd, outbuf, buflen, buf, buflen);
+        {
+          if (cipher_with_keysetup)
+            {
+              err = gcry_cipher_setkey (hd, key, keylen);
+              if (err)
+                {
+                  fprintf (stderr, "gcry_cipher_setkey failed: %s\n",
+                           gpg_strerror (err));
+                  gcry_cipher_close (hd);
+                  exit (1);
+                }
+            }
+          err = gcry_cipher_encrypt ( hd, outbuf, buflen, buf, buflen);
+        }
       stop_timer ();
 
       printf (" %s", elapsed_time ());
       fflush (stdout);
       gcry_cipher_close (hd);
       if (err)
-        { 
+        {
           fprintf (stderr, "gcry_cipher_encrypt failed: %s\n",
                    gpg_strerror (err) );
           exit (1);
@@ -585,25 +686,41 @@ cipher_bench ( const char *algoname )
           fprintf (stderr, PGM ": error opening cipher `%s'/n", algoname);
           exit (1);
         }
-      
-      err = gcry_cipher_setkey (hd, key, keylen);
-      if (err)
-        { 
-          fprintf (stderr, "gcry_cipher_setkey failed: %s\n",
-                   gpg_strerror (err));
-          gcry_cipher_close (hd);
-          exit (1);
+
+      if (!cipher_with_keysetup)
+        {
+          err = gcry_cipher_setkey (hd, key, keylen);
+          if (err)
+            {
+              fprintf (stderr, "gcry_cipher_setkey failed: %s\n",
+                       gpg_strerror (err));
+              gcry_cipher_close (hd);
+              exit (1);
+            }
         }
 
       start_timer ();
       for (i=err=0; !err && i < repetitions; i++)
-        err = gcry_cipher_decrypt ( hd, outbuf, buflen,  buf, buflen);
+        {
+          if (cipher_with_keysetup)
+            {
+              err = gcry_cipher_setkey (hd, key, keylen);
+              if (err)
+                {
+                  fprintf (stderr, "gcry_cipher_setkey failed: %s\n",
+                           gpg_strerror (err));
+                  gcry_cipher_close (hd);
+                  exit (1);
+                }
+            }
+          err = gcry_cipher_decrypt ( hd, outbuf, buflen,  buf, buflen);
+        }
       stop_timer ();
       printf (" %s", elapsed_time ());
       fflush (stdout);
       gcry_cipher_close (hd);
       if (err)
-        { 
+        {
           fprintf (stderr, "gcry_cipher_decrypt failed: %s\n",
                    gpg_strerror (err) );
           exit (1);
@@ -611,8 +728,8 @@ cipher_bench ( const char *algoname )
     }
 
   putchar ('\n');
-  gcry_free (buf);
-  gcry_free (outbuf);
+  gcry_free (raw_buf);
+  gcry_free (raw_outbuf);
 }
 
 
@@ -976,17 +1093,17 @@ mpi_bench (void)
 
   do_powm (
 "20A94417D4D5EF2B2DA99165C7DC87DADB3979B72961AF90D09D59BA24CB9A10166FDCCC9C659F2B9626EC23F3FA425F564A072BA941B03FA81767CC289E4",
-           "29", 
+           "29",
 "B870187A323F1ECD5B8A0B4249507335A1C4CE8394F38FD76B08C78A42C58F6EA136ACF90DFE8603697B1694A3D81114D6117AC1811979C51C4DD013D52F8"
            );
   do_powm (
            "20A94417D4D5EF2B2DA99165C7DC87DADB3979B72961AF90D09D59BA24CB9A10166FDCCC9C659F2B9626EC23F3FA425F564A072BA941B03FA81767CC289E41071F0246879A442658FBD18C1771571E7073EEEB2160BA0CBFB3404D627069A6CFBD53867AD2D9D40231648000787B5C84176B4336144644AE71A403CA40716",
-           "29", 
+           "29",
            "B870187A323F1ECD5B8A0B4249507335A1C4CE8394F38FD76B08C78A42C58F6EA136ACF90DFE8603697B1694A3D81114D6117AC1811979C51C4DD013D52F8FC4EE4BB446B83E48ABED7DB81CBF5E81DE4759E8D68AC985846D999F96B0D8A80E5C69D272C766AB8A23B40D50A4FA889FBC2BD2624222D8EB297F4BAEF8593847"
            );
   do_powm (
            "20A94417D4D5EF2B2DA99165C7DC87DADB3979B72961AF90D09D59BA24CB9A10166FDCCC9C659F2B9626EC23F3FA425F564A072BA941B03FA81767CC289E41071F0246879A442658FBD18C1771571E7073EEEB2160BA0CBFB3404D627069A6CFBD53867AD2D9D40231648000787B5C84176B4336144644AE71A403CA4071620A94417D4D5EF2B2DA99165C7DC87DADB3979B72961AF90D09D59BA24CB9A10166FDCCC9C659F2B9626EC23F3FA425F564A072BA941B03FA81767CC289E41071F0246879A442658FBD18C1771571E7073EEEB2160BA0CBFB3404D627069A6CFBD53867AD2D9D40231648000787B5C84176B4336144644AE71A403CA40716",
-           "29", 
+           "29",
            "B870187A323F1ECD5B8A0B4249507335A1C4CE8394F38FD76B08C78A42C58F6EA136ACF90DFE8603697B1694A3D81114D6117AC1811979C51C4DD013D52F8FC4EE4BB446B83E48ABED7DB81CBF5E81DE4759E8D68AC985846D999F96B0D8A80E5C69D272C766AB8A23B40D50A4FA889FBC2BD2624222D8EB297F4BAEF8593847B870187A323F1ECD5B8A0B4249507335A1C4CE8394F38FD76B08C78A42C58F6EA136ACF90DFE8603697B1694A3D81114D6117AC1811979C51C4DD013D52F8FC4EE4BB446B83E48ABED7DB81CBF5E81DE4759E8D68AC985846D999F96B0D8A80E5C69D272C766AB8A23B40D50A4FA889FBC2BD2624222D8EB297F4BAEF8593847"
            );
 
@@ -1002,6 +1119,9 @@ main( int argc, char **argv )
   int last_argc = -1;
   int no_blinding = 0;
   int use_random_daemon = 0;
+  int with_progress = 0;
+
+  buffer_alignment = 1;
 
   if (argc)
     { argc--; argv++; }
@@ -1041,7 +1161,7 @@ main( int argc, char **argv )
           large_buffers = 1;
           argc--; argv++;
         }
-      else if (!strcmp (*argv, "--cipher-repetition"))
+      else if (!strcmp (*argv, "--cipher-repetitions"))
         {
           argc--; argv++;
           if (argc)
@@ -1050,19 +1170,62 @@ main( int argc, char **argv )
               argc--; argv++;
             }
         }
+      else if (!strcmp (*argv, "--cipher-with-keysetup"))
+        {
+          cipher_with_keysetup = 1;
+          argc--; argv++;
+        }
+      else if (!strcmp (*argv, "--hash-repetitions"))
+        {
+          argc--; argv++;
+          if (argc)
+            {
+              hash_repetitions = atoi(*argv);
+              argc--; argv++;
+            }
+        }
+      else if (!strcmp (*argv, "--alignment"))
+        {
+          argc--; argv++;
+          if (argc)
+            {
+              buffer_alignment = atoi(*argv);
+              argc--; argv++;
+            }
+        }
+      else if (!strcmp (*argv, "--disable-hwf"))
+        {
+          argc--; argv++;
+          if (argc)
+            {
+              if (gcry_control (GCRYCTL_DISABLE_HWF, *argv, NULL))
+                fprintf (stderr, PGM ": unknown hardware feature `%s'"
+                         " - option ignored\n", *argv);
+              argc--; argv++;
+            }
+        }
       else if (!strcmp (*argv, "--fips"))
         {
           argc--; argv++;
           /* This command needs to be called before gcry_check_version.  */
           gcry_control (GCRYCTL_FORCE_FIPS_MODE, 0);
         }
-    }          
+      else if (!strcmp (*argv, "--progress"))
+        {
+          argc--; argv++;
+          with_progress = 1;
+        }
+    }
+
+  if (buffer_alignment < 1 || buffer_alignment > 16)
+    die ("value for --alignment must be in the range 1 to 16\n");
 
   gcry_control (GCRYCTL_SET_VERBOSITY, (int)verbose);
 
   if (!gcry_check_version (GCRYPT_VERSION))
     {
-      fprintf (stderr, PGM ": version mismatch\n");
+      fprintf (stderr, PGM ": version mismatch; pgm=%s, library=%s\n",
+               GCRYPT_VERSION, gcry_check_version (NULL));
       exit (1);
     }
 
@@ -1074,12 +1237,16 @@ main( int argc, char **argv )
   if (use_random_daemon)
     gcry_control (GCRYCTL_USE_RANDOM_DAEMON, 1);
 
-  gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
+  if (with_progress)
+    gcry_set_progress_handler (progress_cb, NULL);
 
+  gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
 
   if (cipher_repetitions < 1)
     cipher_repetitions = 1;
-  
+  if (hash_repetitions < 1)
+    hash_repetitions = 1;
+
   if ( !argc )
     {
       gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
@@ -1152,7 +1319,6 @@ main( int argc, char **argv )
 
   if (in_fips_mode && !gcry_fips_mode_active ())
     fprintf (stderr, PGM ": FIPS mode is not anymore active\n");
-  
+
   return 0;
 }
-
