@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  * MA 02110-1301, USA.
  */
 
@@ -91,7 +91,7 @@ static char *
 mpi2bitstr (gcry_mpi_t a, size_t length)
 {
   char *p, *buf;
-  
+
   buf = p = xmalloc (length+1);
   while (length--)
     *p++ = gcry_mpi_test_bit (a, length) ? '1':'0';
@@ -108,13 +108,20 @@ mpi2bitstr_nlz (gcry_mpi_t a)
 {
   char *p, *buf;
   size_t length = gcry_mpi_get_nbits (a);
-  
-  buf = p = xmalloc (length + 1);
-  while (length-- > 1)
-    *p++ = gcry_mpi_test_bit (a, length) ? '1':'0';
-  *p++ = gcry_mpi_test_bit (a, 0) ? '1':'0';
-  *p = 0;
 
+  if (!length)
+    {
+      buf = p = xmalloc (2);
+      *p++ = '0';
+    }
+  else
+    {
+      buf = p = xmalloc (length + 1);
+      while (length-- > 1)
+        *p++ = gcry_mpi_test_bit (a, length) ? '1':'0';
+      *p++ = gcry_mpi_test_bit (a, 0) ? '1':'0';
+    }
+  *p = 0;
   return buf;
 }
 
@@ -140,7 +147,7 @@ lshiftbitstring (const char *string, size_t n)
 
   if (len+n+1 < len)
     die ("internal overflow\n");
-  /* Allocate enough space. */ 
+  /* Allocate enough space. */
   result = xmalloc (len+n+1);
   for (; *string == '0' && string[1]; string++, len--)
     ;
@@ -190,7 +197,7 @@ one_bit_only (int highbit)
   gcry_mpi_release (a);
 }
 
-/* Check that the shifting actually works for an amount larger than
+/* Check that right shifting actually works for an amount larger than
    the number of bits per limb. */
 static void
 test_rshift (int pass)
@@ -249,7 +256,7 @@ test_rshift (int pass)
   gcry_mpi_release (a);
 }
 
-/* Check that the left shifting.  */
+/* Check that left shifting works correctly.  */
 static void
 test_lshift (int pass)
 {
@@ -275,7 +282,7 @@ test_lshift (int pass)
       for (i=0; i < 75; i++)
         {
           gcry_mpi_lshift (b, a, i);
-          
+
           result = mpi2bitstr_nlz (b);
           tmpstr = mpi2bitstr_nlz (a);
           result2 = lshiftbitstring (tmpstr, i);
@@ -289,11 +296,11 @@ test_lshift (int pass)
           xfree (result);
           xfree (result2);
         }
-      
+
       /* Again. This time using in-place operation. */
       gcry_mpi_randomize (a, size_list[size_idx], GCRY_WEAK_RANDOM);
       gcry_mpi_clear_highbit (a, size_list[size_idx]);
-      
+
       for (i=0; i < 75; i++)
         {
           gcry_mpi_release (b);
@@ -320,6 +327,58 @@ test_lshift (int pass)
 }
 
 
+/* Bug fixed on 2014-05-09:
+      a = gcry_mpi_new (1523);
+      gcry_mpi_set_bit (a, 1536);
+      didn't initialized all limbs in A.  */
+static void
+set_bit_with_resize (void)
+{
+  gcry_mpi_t a;
+  int i;
+
+  wherestr = "set_bit_with_resize";
+  show ("checking that set_bit initializes all limbs\n");
+
+  a = gcry_mpi_new (1536);
+  gcry_mpi_set_bit (a, 1536);
+
+  if (!gcry_mpi_test_bit (a, 1536))
+    fail ("failed to set a bit\n");
+  for (i=0; i < 1536; i++)
+    {
+      if (gcry_mpi_test_bit (a, i))
+        {
+          fail ("spurious bit detected\n");
+          break;
+        }
+    }
+  if (gcry_mpi_test_bit (a, 1537))
+    fail ("more bits set than expected\n");
+  gcry_mpi_release (a);
+
+  wherestr = "set_highbit_with_resize";
+  show ("checking that set_highbit initializes all limbs\n");
+
+  a = gcry_mpi_new (1536);
+  gcry_mpi_set_highbit (a, 1536);
+
+  if (!gcry_mpi_test_bit (a, 1536))
+    fail ("failed to set a bit\n");
+  for (i=0; i < 1536; i++)
+    {
+      if (gcry_mpi_test_bit (a, i))
+        {
+          fail ("spurious bit detected\n");
+          break;
+        }
+    }
+  if (gcry_mpi_test_bit (a, 1537))
+    fail ("more bits set than expected\n");
+  gcry_mpi_release (a);
+}
+
+
 int
 main (int argc, char **argv)
 {
@@ -348,7 +407,9 @@ main (int argc, char **argv)
 
   for (i=0; i < 5; i++)
     test_lshift (i); /* Run several times due to random initializations. */
-  
+
+  set_bit_with_resize ();
+
   show ("All tests completed. Errors: %d\n", error_count);
   return error_count ? 1 : 0;
 }
